@@ -65,6 +65,17 @@ pygments_style = 'sphinx'
 # -- Options for extensions ----------------------------------------------------
 autoclass_content = 'init'
 
+doctest_path = os.path.abspath('..')
+
+doctest_test_doctest_blocks = False
+
+doctest_global_setup = """
+from pymongo.connection import Connection
+connection = Connection()
+connection.drop_database("doctest_test")
+db = connection.doctest_test
+"""
+
 # -- Options for HTML output ---------------------------------------------------
 
 # The theme to use for HTML and HTML Help pages.  Major themes that come with
