@@ -96,7 +96,8 @@ typedef struct {
     int position;
 } bson_buffer;
 
-static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_keys);
+static int write_dict(bson_buffer* buffer, PyObject* dict,
+                      unsigned char check_keys, unsigned char top_level);
 static PyObject* elements_to_dict(const char* string, int max);
 
 static bson_buffer* buffer_new(void) {
@@ -174,7 +175,6 @@ static int buffer_write_bytes(bson_buffer* buffer, const char* bytes, int size)
 
 /* returns 0 on failure */
 static int write_string(bson_buffer* buffer, PyObject* py_string) {
-    int i;
     Py_ssize_t string_length;
     const char* string = PyString_AsString(py_string);
     if (!string) {
@@ -182,13 +182,6 @@ static int write_string(bson_buffer* buffer, PyObject* py_string) {
     }
     string_length = PyString_Size(py_string) + 1;
 
-    for (i = 0; i < string_length - 1; i++) {
-        if (string[i] == 0) {
-            PyErr_SetString(InvalidStringData, "BSON strings must not contain a NULL character");
-            return 0;
-        }
-    }
-
     if (!buffer_write_bytes(buffer, (const char*)&string_length, 4)) {
         return 0;
     }
@@ -238,7 +231,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         return 1;
     } else if (PyDict_Check(value)) {
         *(buffer->buffer + type_byte) = 0x03;
-        return write_dict(buffer, value, check_keys);
+        return write_dict(buffer, value, check_keys, 0);
     } else if (PyList_Check(value) || PyTuple_Check(value)) {
         int start_position,
             length_location,
@@ -382,7 +375,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         if (!scope) {
             return 0;
         }
-        if (!write_dict(buffer, scope, 0)) {
+        if (!write_dict(buffer, scope, 0, 0)) {
             Py_DECREF(scope);
             return 0;
         }
@@ -393,10 +386,12 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         return 1;
     } else if (PyString_Check(value)) {
         int result;
+        result_t status;
 
         *(buffer->buffer + type_byte) = 0x02;
-        if (!is_legal_utf8_string((const unsigned char*)PyString_AsString(value),
-                                  PyString_Size(value))) {
+        status = check_string((const unsigned char*)PyString_AsString(value),
+                              PyString_Size(value), 1, 0);
+        if (status == NOT_UTF_8) {
             PyErr_SetString(InvalidStringData,
                             "strings in documents must be valid UTF-8");
             return 0;
@@ -460,7 +455,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         if (!as_doc) {
             return 0;
         }
-        if (!write_dict(buffer, as_doc, 0)) {
+        if (!write_dict(buffer, as_doc, 0, 0)) {
             Py_DECREF(as_doc);
             return 0;
         }
@@ -475,8 +470,10 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         PyObject* encoded_pattern;
         long int_flags;
         char flags[FLAGS_SIZE];
+        char check_utf8 = 0;
         int pattern_length,
             flags_length;
+        result_t status;
 
         if (!py_flags) {
             return 0;
@@ -496,6 +493,19 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             }
         } else {
             encoded_pattern = py_pattern;
+            check_utf8 = 1;
+        }
+
+        status = check_string((const unsigned char*)PyString_AsString(encoded_pattern),
+                              PyString_Size(encoded_pattern), check_utf8, 1);
+        if (status == NOT_UTF_8) {
+            PyErr_SetString(InvalidStringData,
+                            "regex patterns must be valid UTF-8");
+            return 0;
+        } else if (status == HAS_NULL) {
+            PyErr_SetString(InvalidDocument,
+                            "regex patterns must not contain the NULL byte");
+            return 0;
         }
 
         {
@@ -652,23 +662,40 @@ static int write_pair(bson_buffer* buffer, const char* name, Py_ssize_t name_len
     return 1;
 }
 
-static int decode_and_write_pair(bson_buffer* buffer, PyObject* key,
-                                 PyObject* value, unsigned char check_keys) {
+static int decode_and_write_pair(bson_buffer* buffer,
+                                 PyObject* key, PyObject* value,
+                                 unsigned char check_keys, unsigned char top_level) {
     PyObject* encoded;
     if (PyUnicode_Check(key)) {
+        result_t status;
         encoded = PyUnicode_AsUTF8String(key);
         if (!encoded) {
             return 0;
         }
+        status = check_string((const unsigned char*)PyString_AsString(encoded),
+                              PyString_Size(encoded), 0, 1);
+
+        if (status == HAS_NULL) {
+            PyErr_SetString(InvalidDocument,
+                            "Key names must not contain the NULL byte");
+            return 0;
+        }
     } else if (PyString_Check(key)) {
+        result_t status;
         encoded = key;
         Py_INCREF(encoded);
 
-        if (!is_legal_utf8_string((const unsigned char*)PyString_AsString(encoded),
-                                  PyString_Size(encoded))) {
+        status = check_string((const unsigned char*)PyString_AsString(encoded),
+                                       PyString_Size(encoded), 1, 1);
+
+        if (status == NOT_UTF_8) {
             PyErr_SetString(InvalidStringData,
                             "strings in documents must be valid UTF-8");
             return 0;
+        } else if (status == HAS_NULL) {
+            PyErr_SetString(InvalidDocument,
+                            "Key names must not contain the NULL byte");
+            return 0;
         }
     } else {
         PyObject* errmsg = PyString_FromString("documents must have only string keys, key was ");
@@ -679,9 +706,9 @@ static int decode_and_write_pair(bson_buffer* buffer, PyObject* key,
         return 0;
     }
 
-    /* Don't allow writing _id here - it was already written. */
+    /* If top_level is True, don't allow writing _id here - it was already written. */
     if (!write_pair(buffer, PyString_AsString(encoded),
-                    PyString_Size(encoded), value, check_keys, 0)) {
+                    PyString_Size(encoded), value, check_keys, !top_level)) {
         Py_DECREF(encoded);
         return 0;
     }
@@ -691,7 +718,8 @@ static int decode_and_write_pair(bson_buffer* buffer, PyObject* key,
 }
 
 static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
-                     int length_location, unsigned char check_keys) {
+                     int length_location, unsigned char check_keys,
+                     unsigned char top_level) {
     PyObject* keys = PyObject_CallMethod(dict, "keys", NULL);
     int items,
         i;
@@ -710,7 +738,7 @@ static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
         }
         value = PyDict_GetItem(dict, key);
         if (!value ||
-            !decode_and_write_pair(buffer, key, value, check_keys)) {
+            !decode_and_write_pair(buffer, key, value, check_keys, top_level)) {
             Py_DECREF(keys);
             return 0;
         }
@@ -720,7 +748,7 @@ static int write_son(bson_buffer* buffer, PyObject* dict, int start_position,
 }
 
 /* returns 0 on failure */
-static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_keys) {
+static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_keys, unsigned char top_level) {
     int start_position = buffer->position;
     char zero = 0;
     int length;
@@ -733,8 +761,8 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
         return 0;
     }
 
-    /* Write _id first if we have one, whether or not we're SON. */
-    if (is_dict) {
+    /* Write _id first if this is a top level doc. */
+    if (is_dict && top_level) {
         PyObject* _id = PyDict_GetItemString(dict, "_id");
         if (_id) {
             /* Don't bother checking keys, but do make sure we're allowed to
@@ -746,7 +774,7 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
     }
 
     if (PyObject_IsInstance(dict, SON)) {
-        if (!write_son(buffer, dict, start_position, length_location, check_keys)) {
+        if (!write_son(buffer, dict, start_position, length_location, check_keys, top_level)) {
             return 0;
         }
     } else if (is_dict) {
@@ -755,7 +783,7 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
         Py_ssize_t pos = 0;
 
         while (PyDict_Next(dict, &pos, &key, &value)) {
-            if (!decode_and_write_pair(buffer, key, value, check_keys)) {
+            if (!decode_and_write_pair(buffer, key, value, check_keys, top_level)) {
                 return 0;
             }
         }
@@ -797,7 +825,7 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
         return NULL;
     }
 
-    if (!write_dict(buffer, dict, check_keys)) {
+    if (!write_dict(buffer, dict, check_keys, 1)) {
         buffer_free(buffer);
         return NULL;
     }
@@ -834,7 +862,7 @@ static int add_last_error(bson_buffer* buffer, int request_id) {
 static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     /* NOTE just using a random number as the request_id */
     int request_id = rand();
-    char* collection_name;
+    char* collection_name = NULL;
     int collection_name_length;
     PyObject* docs;
     int i;
@@ -844,7 +872,8 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     int length_location;
     PyObject* result;
 
-    if (!PyArg_ParseTuple(args, "s#Obb",
+    if (!PyArg_ParseTuple(args, "et#Obb",
+                          "utf-8",
                           &collection_name,
                           &collection_name_length,
                           &docs, &check_keys, &safe)) {
@@ -853,6 +882,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
 
     buffer = buffer_new();
     if (!buffer) {
+        PyMem_Free(collection_name);
         return NULL;
     }
 
@@ -868,13 +898,16 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
         !buffer_write_bytes(buffer,
                             collection_name,
                             collection_name_length + 1)) {
+        PyMem_Free(collection_name);
         buffer_free(buffer);
         return NULL;
     }
 
+    PyMem_Free(collection_name);
+
     for (i = 0; i < PyList_Size(docs); i++) {
         PyObject* doc = PyList_GetItem(docs, i);
-        if (!write_dict(buffer, doc, check_keys)) {
+        if (!write_dict(buffer, doc, check_keys, 1)) {
             buffer_free(buffer);
             return NULL;
         }
@@ -896,6 +929,80 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     return result;
 }
 
+static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
+    /* NOTE just using a random number as the request_id */
+    int request_id = rand();
+    char* collection_name = NULL;
+    int collection_name_length;
+    PyObject* doc;
+    PyObject* spec;
+    unsigned char multi;
+    unsigned char upsert;
+    unsigned char safe;
+    int options;
+    bson_buffer* buffer;
+    int length_location;
+    PyObject* result;
+
+    if (!PyArg_ParseTuple(args, "et#bbOOb",
+                          "utf-8",
+                          &collection_name,
+                          &collection_name_length,
+                          &upsert, &multi, &spec, &doc, &safe)) {
+        return NULL;
+    }
+
+    options = 0;
+    if (upsert) {
+        options += 1;
+    }
+    if (multi) {
+        options += 2;
+    }
+    buffer = buffer_new();
+    if (!buffer) {
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    // save space for message length
+    length_location = buffer_save_bytes(buffer, 4);
+    if (length_location == -1 ||
+        !buffer_write_bytes(buffer, (const char*)&request_id, 4) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"
+                            "\xd1\x07\x00\x00"
+                            "\x00\x00\x00\x00",
+                            12) ||
+        !buffer_write_bytes(buffer,
+                            collection_name,
+                            collection_name_length + 1) ||
+        !buffer_write_bytes(buffer, (const char*)&options, 4) ||
+        !write_dict(buffer, spec, 0, 1) ||
+        !write_dict(buffer, doc, 0, 1)) {
+        buffer_free(buffer);
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    PyMem_Free(collection_name);
+
+    memcpy(buffer->buffer + length_location, &buffer->position, 4);
+
+    if (safe) {
+        if (!add_last_error(buffer, request_id)) {
+            buffer_free(buffer);
+            return NULL;
+        }
+    }
+
+    /* objectify buffer */
+    result = Py_BuildValue("is#", request_id,
+                           buffer->buffer, buffer->position);
+    buffer_free(buffer);
+    return result;
+}
+
 static PyObject* get_value(const char* buffer, int* position, int type) {
     PyObject* value;
     switch (type) {
@@ -1315,6 +1422,8 @@ static PyMethodDef _CBSONMethods[] = {
      "convert binary data to a sequence of SON objects."},
     {"_insert_message", _cbson_insert_message, METH_VARARGS,
      "create an insert message to be sent to MongoDB"},
+    {"_update_message", _cbson_update_message, METH_VARARGS,
+     "create an update message to be sent to MongoDB"},
     {NULL, NULL, 0, NULL}
 };
 
