@@ -16,19 +16,18 @@
 
 Generally not needed to be used by application developers."""
 
-import types
 import struct
 import re
 import datetime
 import calendar
 
-from binary import Binary
-from code import Code
-from objectid import ObjectId
-from dbref import DBRef
-from son import SON
-from errors import InvalidBSON, InvalidDocument
-from errors import InvalidName, InvalidStringData
+from pymongo.binary import Binary
+from pymongo.code import Code
+from pymongo.objectid import ObjectId
+from pymongo.dbref import DBRef
+from pymongo.son import SON
+from pymongo.errors import InvalidBSON, InvalidDocument
+from pymongo.errors import InvalidName, InvalidStringData
 
 try:
     import _cbson
@@ -52,18 +51,20 @@ def _get_int(data):
     return (value, data[4:])
 
 
-def _get_c_string(data):
-    try:
-        end = data.index("\x00")
-    except ValueError:
-        raise InvalidBSON()
+def _get_c_string(data, length=None):
+    if length is None:
+        try:
+            length = data.index("\x00")
+        except ValueError:
+            raise InvalidBSON()
 
-    return (unicode(data[:end], "utf-8"), data[end + 1:])
+    return (unicode(data[:length], "utf-8"), data[length + 1:])
 
 
-def _make_c_string(string):
-    if "\x00" in string:
-        raise InvalidStringData("BSON strings must not contain a NULL character")
+def _make_c_string(string, check_null=False):
+    if check_null and "\x00" in string:
+        raise InvalidDocument("BSON keys / regex patterns must not "
+                              "contain a NULL character")
     if isinstance(string, unicode):
         return string.encode("utf-8") + "\x00"
     else:
@@ -235,13 +236,14 @@ def _get_number(data):
 
 
 def _get_string(data):
-    return _get_c_string(data[4:])
+    return _get_c_string(data[4:], struct.unpack("<i", data[:4])[0] - 1)
 
 
 def _get_object(data):
     (object, data) = _bson_to_dict(data)
     if "$ref" in object:
-        return (DBRef(object["$ref"], object["$id"], object.get("$db", None)), data)
+        return (DBRef(object["$ref"], object["$id"],
+                      object.get("$db", None)), data)
     return (object, data)
 
 
@@ -378,8 +380,9 @@ _RE_TYPE = type(_valid_array_name)
 
 
 def _element_to_bson(key, value, check_keys):
-    if not isinstance(key, (str, unicode)):
-        raise InvalidDocument("documents must have only string keys, key was %r" % key)
+    if not isinstance(key, basestring):
+        raise InvalidDocument("documents must have only string keys, "
+                              "key was %r" % key)
 
     if check_keys:
         if key.startswith("$"):
@@ -387,7 +390,7 @@ def _element_to_bson(key, value, check_keys):
         if "." in key:
             raise InvalidName("key %r must not contain '.'" % key)
 
-    name = _make_c_string(key)
+    name = _make_c_string(key, True)
     if isinstance(value, float):
         return "\x01" + name + struct.pack("<d", value)
 
@@ -408,7 +411,7 @@ def _element_to_bson(key, value, check_keys):
                                  chr(subtype), value)
     if isinstance(value, Code):
         cstring = _make_c_string(value)
-        scope = _dict_to_bson(value.scope, False)
+        scope = _dict_to_bson(value.scope, False, False)
         full_length = struct.pack("<i", 8 + len(cstring) + len(scope))
         length = struct.pack("<i", len(cstring))
         return "\x0F" + name + full_length + length + cstring + scope
@@ -421,10 +424,10 @@ def _element_to_bson(key, value, check_keys):
         length = struct.pack("<i", len(cstring))
         return "\x02" + name + length + cstring
     if isinstance(value, dict):
-        return "\x03" + name + _dict_to_bson(value, check_keys)
+        return "\x03" + name + _dict_to_bson(value, check_keys, False)
     if isinstance(value, (list, tuple)):
         as_dict = SON(zip([str(i) for i in range(len(value))], value))
-        return "\x04" + name + _dict_to_bson(as_dict, check_keys)
+        return "\x04" + name + _dict_to_bson(as_dict, check_keys, False)
     if isinstance(value, ObjectId):
         return "\x07" + name + value.binary
     if value is True:
@@ -459,7 +462,8 @@ def _element_to_bson(key, value, check_keys):
             flags += "u"
         if value.flags & re.VERBOSE:
             flags += "x"
-        return "\x0B" + name + _make_c_string(pattern) + _make_c_string(flags)
+        return "\x0B" + name + _make_c_string(pattern, True) + \
+            _make_c_string(flags)
     if isinstance(value, DBRef):
         return _element_to_bson(key, value.as_doc(), False)
 
@@ -467,13 +471,13 @@ def _element_to_bson(key, value, check_keys):
                           type(value))
 
 
-def _dict_to_bson(dict, check_keys):
+def _dict_to_bson(dict, check_keys, top_level=True):
     try:
         elements = ""
-        if "_id" in dict:
+        if top_level and "_id" in dict:
             elements += _element_to_bson("_id", dict["_id"], False)
         for (key, value) in dict.iteritems():
-            if key != "_id":
+            if not top_level or key != "_id":
                 elements += _element_to_bson(key, value, check_keys)
     except AttributeError:
         raise TypeError("encoder expected a mapping type but got: %r" % dict)
@@ -510,15 +514,16 @@ def _to_dict(data):
 
 
 def is_valid(bson):
-    """Validate that the given string represents valid BSON data.
+    """Check that the given string represents valid :class:`BSON` data.
 
-    Raises TypeError if the data is not an instance of a subclass of str.
-    Returns True if the data represents a valid BSON object, False otherwise.
+    Raises :class:`TypeError` if `bson` is not an instance of
+    :class:`str`.  Returns ``True`` if `bson` is valid :class:`BSON`,
+    ``False`` otherwise.
 
     :Parameters:
       - `bson`: the data to be validated
     """
-    if not isinstance(bson, types.StringType):
+    if not isinstance(bson, str):
         raise TypeError("BSON data must be an instance of a subclass of str")
 
     # 4 MB limit
@@ -539,28 +544,32 @@ class BSON(str):
     """
 
     def __new__(cls, bson):
-        """Initialize a new BSON object with some data.
+        """Initialize a new :class:`BSON` instance with some data.
 
-        Raises TypeError if `bson` is not an instance of str.
+        Raises :class:`TypeError` if `bson` is not an instance of
+        :class:`str`.
 
         :Parameters:
           - `bson`: the initial data
         """
         return str.__new__(cls, bson)
 
-    def from_dict(cls, dict, check_keys=False):
-        """Create a new BSON object from a python mapping type (like dict).
+    def from_dict(cls, dct, check_keys=False):
+        """Create a new :class:`BSON` instance from a mapping type
+        (like :class:`dict`).
 
-        Raises TypeError if the argument is not a mapping type, or contains
-        keys that are not instance of (str, unicode). Raises InvalidDocument
-        if the dictionary cannot be converted to BSON.
+        Raises :class:`TypeError` if `dct` is not a mapping type, or
+        contains keys that are not instances of :class:`basestring`.
+        Raises :class:`~pymongo.errors.InvalidDocument` if `dct`
+        cannot be converted to :class:`BSON`.
 
         :Parameters:
-          - `dict`: mapping type representing a Mongo document
-          - `check_keys`: check if keys start with '$' or contain '.',
-            raising `pymongo.errors.InvalidName` in either case
+          - `dct`: mapping type representing a document
+          - `check_keys` (optional): check if keys start with '$' or
+            contain '.', raising :class:`~pymongo.errors.InvalidName`
+            in either case
         """
-        return cls(_dict_to_bson(dict, check_keys))
+        return cls(_dict_to_bson(dct, check_keys))
     from_dict = classmethod(from_dict)
 
     def to_dict(self):
