@@ -15,14 +15,12 @@
 """Tools for representing JavaScript code to be evaluated by MongoDB.
 """
 
-import types
-
-
 class Code(str):
     """JavaScript code to be evaluated by MongoDB.
 
-    Raises TypeError if `code` is not an instance of (str, unicode) or
-    `scope` is not an instance of dict.
+    Raises :class:`TypeError` if `code` is not an instance of
+    :class:`basestring` or `scope` is not an instance of
+    :class:`dict`.
 
     :Parameters:
       - `code`: string containing JavaScript code to be evaluated
@@ -32,12 +30,15 @@ class Code(str):
     """
 
     def __new__(cls, code, scope=None):
-        if not isinstance(code, types.StringTypes):
-            raise TypeError("code must be an instance of (str, unicode)")
+        if not isinstance(code, basestring):
+            raise TypeError("code must be an instance of basestring")
 
         if scope is None:
-            scope = {}
-        if not isinstance(scope, types.DictType):
+            try:
+                scope = code.scope
+            except AttributeError:
+                scope = {}
+        if not isinstance(scope, dict):
             raise TypeError("scope must be an instance of dict")
 
         self = str.__new__(cls, code)
@@ -52,3 +53,8 @@ class Code(str):
 
     def __repr__(self):
         return "Code(%s, %r)" % (str.__repr__(self), self.__scope)
+
+    def __eq__(self, other):
+        if isinstance(other, Code):
+            return (self.__scope, str(self)) == (other.__scope, str(other))
+        return False
