@@ -14,7 +14,6 @@
 
 """Collection level utilities for Mongo."""
 
-import types
 import warnings
 import struct
 
@@ -23,7 +22,7 @@ import message
 from objectid import ObjectId
 from cursor import Cursor
 from son import SON
-from errors import InvalidName, OperationFailure
+from errors import InvalidName
 from code import Code
 
 _ZERO = "\x00\x00\x00\x00"
@@ -36,22 +35,25 @@ class Collection(object):
     def __init__(self, database, name, options=None):
         """Get / create a Mongo collection.
 
-        Raises TypeError if name is not an instance of (str, unicode). Raises
-        InvalidName if name is not a valid collection name. Raises TypeError if
-        options is not an instance of dict. If options is non-empty a create
-        command will be sent to the database. Otherwise the collection will be
-        created implicitly on first use.
+        Raises :class:`TypeError` if `name` is not an instance of
+        :class:`basestring`. Raises
+        :class:`~pymongo.errors.InvalidName` if `name` is not a valid
+        collection name. Raises :class:`TypeError` if `options` is not
+        an instance of :class:`dict`. If `options` is non-empty a
+        create command will be sent to the database. Otherwise the
+        collection will be created implicitly on first use.
 
         :Parameters:
           - `database`: the database to get a collection from
           - `name`: the name of the collection to get
-          - `options`: dictionary of collection options.
-            see `pymongo.database.Database.create_collection` for details.
+          - `options`: dictionary of collection options.  see
+            :meth:`~pymongo.database.Database.create_collection` for
+            details.
         """
-        if not isinstance(name, types.StringTypes):
-            raise TypeError("name must be an instance of (str, unicode)")
+        if not isinstance(name, basestring):
+            raise TypeError("name must be an instance of basestring")
 
-        if not isinstance(options, (types.DictType, types.NoneType)):
+        if options is not None and not isinstance(options, dict):
             raise TypeError("options must be an instance of dict")
 
         if not name or ".." in name:
@@ -63,6 +65,9 @@ class Collection(object):
         if name[0] == "." or name[-1] == ".":
             raise InvalidName("collecion names must not start "
                               "or end with '.': %r" % name)
+        if "\x00" in name:
+            raise InvalidName("collection names must not contain the "
+                              "null character")
 
         self.__database = database
         self.__name = unicode(name)
@@ -89,7 +94,7 @@ class Collection(object):
         command = SON({"create": self.__name})
         command.update(options)
 
-        self.__database._command(command)
+        self.__database.command(command)
 
     def __getattr__(self, name):
         """Get a sub-collection of this collection by name.
@@ -165,7 +170,7 @@ class Collection(object):
           - `manipulate` (optional): manipulate the SON object before saving it
           - `safe` (optional): check that the save succeeded?
         """
-        if not isinstance(to_save, types.DictType):
+        if not isinstance(to_save, dict):
             raise TypeError("cannot save object of type %s" % type(to_save))
 
         if "_id" not in to_save:
@@ -198,7 +203,7 @@ class Collection(object):
            Bulk insert works with any iterable
         """
         docs = doc_or_docs
-        if isinstance(docs, types.DictType):
+        if isinstance(docs, dict):
             docs = [docs]
 
         if manipulate:
@@ -234,6 +239,9 @@ class Collection(object):
           >>> list(db.test.find())
           [{u'a': u'c', u'x': u'y', u'_id': ObjectId('...')}]
 
+        If `safe` is ``True`` returns the response to the *lastError*
+        command. Otherwise, returns ``None``.
+
         :Parameters:
           - `spec`: a ``dict`` or :class:`~pymongo.son.SON` instance specifying
             elements which must be present for a document to be updated
@@ -250,22 +258,24 @@ class Collection(object):
             explicitly for all update operations in order to prepare your code
             for that change.
 
+        .. versionchanged:: 1.4
+           Return the response to *lastError* if `safe` is ``True``.
         .. versionadded:: 1.1.1
            The `multi` parameter.
 
         .. _update modifiers: http://www.mongodb.org/display/DOCS/Updating
         """
-        if not isinstance(spec, types.DictType):
+        if not isinstance(spec, dict):
             raise TypeError("spec must be an instance of dict")
-        if not isinstance(document, types.DictType):
+        if not isinstance(document, dict):
             raise TypeError("document must be an instance of dict")
-        if not isinstance(upsert, types.BooleanType):
+        if not isinstance(upsert, bool):
             raise TypeError("upsert must be an instance of bool")
 
         if upsert and manipulate:
             document = self.__database._fix_incoming(document, self)
 
-        self.__database.connection._send_message(
+        return self.__database.connection._send_message(
             message.update(self.__full_name, upsert, multi,
                            spec, document, safe), safe)
 
@@ -288,6 +298,9 @@ class Collection(object):
         :meth:`~pymongo.database.Database.drop_collection`, however, as
         indexes will not be removed.
 
+        If `safe` is ``True`` returns the response to the *lastError*
+        command. Otherwise, returns ``None``.
+
         :Parameters:
           - `spec_or_object_id` (optional): a ``dict`` or
             :class:`~pymongo.son.SON` instance specifying which documents
@@ -296,11 +309,12 @@ class Collection(object):
             ``_id`` field for the document to be removed
           - `safe` (optional): check that the remove succeeded?
 
+        .. versionchanged:: 1.4
+           Return the response to *lastError* if `safe` is ``True``.
         .. versionchanged:: 1.2
            The `spec_or_object_id` parameter is now optional. If it is
            not specified *all* documents in the collection will be
            removed.
-
         .. versionadded:: 1.1
            The `safe` parameter.
         """
@@ -310,15 +324,15 @@ class Collection(object):
         if isinstance(spec, ObjectId):
             spec = {"_id": spec}
 
-        if not isinstance(spec, types.DictType):
+        if not isinstance(spec, dict):
             raise TypeError("spec must be an instance of dict, not %s" %
                             type(spec))
 
-        self.__database.connection._send_message(
+        return self.__database.connection._send_message(
             message.delete(self.__full_name, spec, safe), safe)
 
-    def find_one(self, spec_or_object_id=None, fields=None, slave_okay=None,
-                 _sock=None, _must_use_master=False):
+    def find_one(self, spec_or_object_id=None, fields=None,
+                 _sock=None, _must_use_master=False, _is_command=False):
         """Get a single object from the database.
 
         Raises TypeError if the argument is of an improper type. Returns a
@@ -330,9 +344,6 @@ class Collection(object):
             of ObjectId to be used as the value for an _id query
           - `fields` (optional): a list of field names that should be included
             in the returned document ("_id" will always be included)
-          - `slave_okay` (optional): DEPRECATED this option is deprecated and
-            will be removed - see the slave_okay parameter to
-            `pymongo.Connection.__init__`.
         """
         spec = spec_or_object_id
         if spec is None:
@@ -341,8 +352,8 @@ class Collection(object):
             spec = SON({"_id": spec})
 
         for result in self.find(spec, limit=-1, fields=fields,
-                                slave_okay=slave_okay, _sock=_sock,
-                                _must_use_master=_must_use_master):
+                                _sock=_sock, _must_use_master=_must_use_master,
+                                _is_command=_is_command):
             return result
         return None
 
@@ -357,15 +368,15 @@ class Collection(object):
         """
         as_dict = {}
         for field in fields:
-            if not isinstance(field, types.StringTypes):
+            if not isinstance(field, basestring):
                 raise TypeError("fields must be a list of key names as "
                                 "(string, unicode)")
             as_dict[field] = 1
         return as_dict
 
     def find(self, spec=None, fields=None, skip=0, limit=0,
-             slave_okay=None, timeout=True, snapshot=False, tailable=False,
-             _sock=None, _must_use_master=False):
+             timeout=True, snapshot=False, tailable=False,
+             _sock=None, _must_use_master=False, _is_command=False):
         """Query the database.
 
         The `spec` argument is a prototype document that all results must
@@ -390,9 +401,6 @@ class Collection(object):
           - `skip` (optional): the number of documents to omit (from the start
             of the result set) when returning the results
           - `limit` (optional): the maximum number of results to return
-          - `slave_okay` (optional): DEPRECATED this option is deprecated and
-            will be removed - see the slave_okay parameter to
-            `pymongo.Connection.__init__`.
           - `timeout` (optional): if True, any returned cursor will be subject
             to the normal timeout behavior of the mongod process. Otherwise,
             the returned cursor will never timeout at the server. Care should
@@ -416,29 +424,22 @@ class Collection(object):
         """
         if spec is None:
             spec = SON()
-        if slave_okay is None:
-            slave_okay = self.__database.connection.slave_okay
-        else:
-            warnings.warn("The slave_okay option to find and find_one is "
-                          "deprecated. Please set slave_okay on the Connection "
-                          "itself.",
-                          DeprecationWarning)
 
-        if not isinstance(spec, types.DictType):
+        slave_okay = self.__database.connection.slave_okay
+
+        if not isinstance(spec, dict):
             raise TypeError("spec must be an instance of dict")
-        if not isinstance(fields, (types.ListType, types.NoneType)):
+        if fields is not None and not isinstance(fields, list):
             raise TypeError("fields must be an instance of list")
-        if not isinstance(skip, types.IntType):
+        if not isinstance(skip, int):
             raise TypeError("skip must be an instance of int")
-        if not isinstance(limit, types.IntType):
+        if not isinstance(limit, int):
             raise TypeError("limit must be an instance of int")
-        if not isinstance(slave_okay, types.BooleanType):
-            raise TypeError("slave_okay must be an instance of bool")
-        if not isinstance(timeout, types.BooleanType):
+        if not isinstance(timeout, bool):
             raise TypeError("timeout must be an instance of bool")
-        if not isinstance(snapshot, types.BooleanType):
+        if not isinstance(snapshot, bool):
             raise TypeError("snapshot must be an instance of bool")
-        if not isinstance(tailable, types.BooleanType):
+        if not isinstance(tailable, bool):
             raise TypeError("tailable must be an instance of bool")
 
         if fields is not None:
@@ -448,7 +449,8 @@ class Collection(object):
 
         return Cursor(self, spec, fields, skip, limit, slave_okay, timeout,
                       tailable, snapshot, _sock=_sock,
-                      _must_use_master=_must_use_master)
+                      _must_use_master=_must_use_master,
+                      _is_command=_is_command)
 
     def count(self):
         """Get the number of documents in this collection.
@@ -463,32 +465,28 @@ class Collection(object):
         """
         return u"_".join([u"%s_%s" % item for item in keys])
 
-    def create_index(self, key_or_list, direction=None, unique=False, ttl=300):
+    def create_index(self, key_or_list, unique=False, ttl=300):
         """Creates an index on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of ``(str, unicode)``, and the
+        The key(s) must be an instance of :class:`basestring`, and the
         directions must be one of (:data:`~pymongo.ASCENDING`,
         :data:`~pymongo.DESCENDING`). Returns the name of the created
         index.
 
         :Parameters:
-          - `key_or_list`: a single key or a list of (key, direction) pairs
-            specifying the index to create
-          - `direction` (optional): DEPRECATED this option will be removed
-          - `unique` (optional): should this index guarantee uniqueness?
-          - `ttl` (optional): time window (in seconds) during which this index
-            will be recognized by subsequent calls to :meth:`ensure_index` -
-            see documentation for :meth:`ensure_index` for details
+          - `key_or_list`: a single key or a list of (key, direction)
+            pairs specifying the index to create
+          - `unique` (optional): should this index guarantee
+            uniqueness?
+          - `ttl` (optional): time window (in seconds) during which
+            this index will be recognized by subsequent calls to
+            :meth:`ensure_index` - see documentation for
+            :meth:`ensure_index` for details
         """
         if not isinstance(key_or_list, (str, unicode, list)):
-            raise TypeError("key_or_list must either be a single key or a list of (key, direction) pairs")
-
-        if direction is not None:
-            warnings.warn("specifying a direction for a single key index is "
-                          "deprecated and will be removed. there is no need "
-                          "for a direction on a single key index",
-                          DeprecationWarning)
+            raise TypeError("key_or_list must either be a single key "
+                            "or a list of (key, direction) pairs")
 
         to_save = SON()
         keys = helpers._index_list(key_or_list)
@@ -505,13 +503,13 @@ class Collection(object):
                                               check_keys=False)
         return to_save["name"]
 
-    def ensure_index(self, key_or_list, direction=None, unique=False, ttl=300):
+    def ensure_index(self, key_or_list, unique=False, ttl=300):
         """Ensures that an index exists on this collection.
 
-        Takes either a single key or a list of (key, direction)
-        pairs.  The key(s) must be an instance of ``(str, unicode)``,
-        and the direction(s) must be one of
-        (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`).
+        Takes either a single key or a list of (key, direction) pairs.
+        The key(s) must be an instance of :class:`basestring`, and the
+        direction(s) must be one of (:data:`~pymongo.ASCENDING`,
+        :data:`~pymongo.DESCENDING`).
 
         Unlike :meth:`create_index`, which attempts to create an index
         unconditionally, :meth:`ensure_index` takes advantage of some
@@ -532,22 +530,17 @@ class Collection(object):
         created. Returns ``None`` if the index already exists.
 
         :Parameters:
-          - `key_or_list`: a single key or a list of (key, direction) pairs
-            specifying the index to ensure
-          - `direction` (optional): DEPRECATED this option will be removed
-          - `unique` (optional): should this index guarantee uniqueness?
-          - `ttl` (optional): time window (in seconds) during which this index
-            will be recognized by subsequent calls to :meth:`ensure_index`
+          - `key_or_list`: a single key or a list of (key, direction)
+            pairs specifying the index to ensure
+          - `unique` (optional): should this index guarantee
+            uniqueness?
+          - `ttl` (optional): time window (in seconds) during which
+            this index will be recognized by subsequent calls to
+            :meth:`ensure_index`
         """
         if not isinstance(key_or_list, (str, unicode, list)):
             raise TypeError("key_or_list must either be a single key or a list of (key, direction) pairs")
 
-        if direction is not None:
-            warnings.warn("specifying a direction for a single key index is "
-                          "deprecated and will be removed. there is no need "
-                          "for a direction on a single key index",
-                          DeprecationWarning)
-
         keys = helpers._index_list(key_or_list)
         name = self._gen_index_name(keys)
         if self.__database.connection._cache_index(self.__database.name,
@@ -579,18 +572,18 @@ class Collection(object):
           - `index_or_name`: index (or name of index) to drop
         """
         name = index_or_name
-        if isinstance(index_or_name, types.ListType):
+        if isinstance(index_or_name, list):
             name = self._gen_index_name(index_or_name)
 
-        if not isinstance(name, types.StringTypes):
+        if not isinstance(name, basestring):
             raise TypeError("index_or_name must be an index name or list")
 
         self.__database.connection._purge_index(self.__database.name,
                                                 self.__name, name)
-        self.__database._command(SON([("deleteIndexes",
-                                       self.__name),
-                                      ("index", name)]),
-                                 ["ns not found"])
+        self.__database.command(SON([("deleteIndexes",
+                                      self.__name),
+                                     ("index", name)]),
+                                ["ns not found"])
 
     def index_information(self):
         """Get information on this collection's indexes.
@@ -625,20 +618,26 @@ class Collection(object):
 
         return options
 
-    # TODO send all groups as commands once 1.2 is out
-    #
-    # Waiting on this because group command support for CodeWScope
-    # wasn't added until 1.1
-    def group(self, keys, condition, initial, reduce, finalize=None,
+    # TODO key and condition ought to be optional, but deprecation
+    # could be painful as argument order would have to change.
+    def group(self, key, condition, initial, reduce, finalize=None,
               command=True):
-        """Perform a query similar to an SQL group by operation.
+        """Perform a query similar to an SQL *group by* operation.
 
         Returns an array of grouped items.
 
+        The `key` parameter can be:
+
+          - ``None`` to use the entire document as a key.
+          - A :class:`list` of keys (each a :class:`basestring`) to group by.
+          - A :class:`basestring` or :class:`~pymongo.code.Code` instance
+            containing a JavaScript function to be applied to each document,
+            returning the key to group by.
+
         :Parameters:
-          - `keys`: list of fields to group by
-          - `condition`: specification of rows to be considered (as a `find`
-            query specification)
+          - `key`: fields to group by (see above description)
+          - `condition`: specification of rows to be
+            considered (as a :meth:`find` query specification)
           - `initial`: initial value of the aggregation counter object
           - `reduce`: aggregation function as a JavaScript string
           - `finalize`: function to be called on each object in output list.
@@ -646,91 +645,45 @@ class Collection(object):
             command instead of in an eval - this option is deprecated and
             will be removed in favor of running all groups as commands
 
+        .. versionchanged:: 1.4
+           The `key` argument can now be ``None`` or a JavaScript function,
+           in addition to a :class:`list` of keys.
         .. versionchanged:: 1.3
            The `command` argument now defaults to ``True`` and is deprecated.
         """
-
-        #for now support people passing command in its old position
-        if finalize in (True, False):
-            command = finalize
-            finalize = None
-            warnings.warn("Please only pass 'command' as a keyword argument.",
+        if not command:
+            warnings.warn("eval-based groups are deprecated, and the "
+                          "command option will be removed.",
                           DeprecationWarning)
 
-        if command:
-            if not isinstance(reduce, Code):
-                reduce = Code(reduce)
-            group = {"ns": self.__name,
-                    "$reduce": reduce,
-                    "key": self._fields_list_to_dict(keys),
-                    "cond": condition,
-                    "initial": initial}
-            if finalize is not None:
-                if not isinstance(finalize, Code):
-                    finalize = Code(finalize)
-                group["finalize"] = finalize
-            return self.__database._command({"group":group})["retval"]
+        group = {}
+        if isinstance(key, basestring):
+            group["$keyf"] = Code(key)
+        elif key is not None:
+            group = {"key": self._fields_list_to_dict(key)}
+        group["ns"] = self.__name
+        group["$reduce"] = Code(reduce)
+        group["cond"] = condition
+        group["initial"] = initial
+        if finalize is not None:
+            group["finalize"] = Code(finalize)
 
-        warnings.warn("eval-based groups are deprecated. please use command=True.",
-                      DeprecationWarning)
-
-        scope = {}
-        if isinstance(reduce, Code):
-            scope = reduce.scope
-        scope.update({"ns": self.__name,
-                      "keys": keys,
-                      "condition": condition,
-                      "initial": initial})
-
-        group_function = """function () {
-    var c = db[ns].find(condition);
-    var map = new Map();
-    var reduce_function = %s;
-    var finalize_function = %s; //function or null
-    while (c.hasNext()) {
-        var obj = c.next();
-
-        var key = {};
-        for (var i = 0; i < keys.length; i++) {
-            var k = keys[i];
-            key[k] = obj[k];
-        }
-
-        var aggObj = map.get(key);
-        if (aggObj == null) {
-            var newObj = Object.extend({}, key);
-            aggObj = Object.extend(newObj, initial);
-            map.put(key, aggObj);
-        }
-        reduce_function(obj, aggObj);
-    }
-
-    out = map.values();
-    if (finalize_function !== null){
-        for (var i=0; i < out.length; i++){
-            var ret = finalize_function(out[i]);
-            if (ret !== undefined)
-                out[i] = ret;
-        }
-    }
-
-    return {"result": out};
-}""" % (reduce, (finalize or 'null'));
-        return self.__database.eval(Code(group_function, scope))["result"]
+        return self.__database.command({"group":group})["retval"]
 
     def rename(self, new_name):
         """Rename this collection.
 
-        If operating in auth mode, client must be authorized as an admin to
-        perform this operation. Raises TypeError if new_name is not an instance
-        of (str, unicode). Raises InvalidName if new_name is not a valid
-        collection name.
+        If operating in auth mode, client must be authorized as an
+        admin to perform this operation. Raises :class:`TypeError` if
+        `new_name` is not an instance of :class:`basestring`. Raises
+        :class:`~pymongo.errors.InvalidName` if `new_name` is not a
+        valid collection name.
 
         :Parameters:
           - `new_name`: new name for this collection
         """
-        if not isinstance(new_name, types.StringTypes):
-            raise TypeError("new_name must be an instance of (str, unicode)")
+        if not isinstance(new_name, basestring):
+            raise TypeError("new_name must be an instance of basestring")
 
         if not new_name or ".." in new_name:
             raise InvalidName("collection names cannot be empty")
@@ -743,17 +696,17 @@ class Collection(object):
                               ("to", "%s.%s" % (self.__database.name,
                                                 new_name))])
 
-        self.__database.connection.admin._command(rename_command)
+        self.__database.connection.admin.command(rename_command)
 
     def distinct(self, key):
-        """Get a list of distinct values for `key` among all documents in this
-        collection.
+        """Get a list of distinct values for `key` among all documents
+        in this collection.
 
         Raises :class:`TypeError` if `key` is not an instance of
-        ``(str, unicode)``.
+        :class:`basestring`.
 
-        To get the distinct values for a key in the result set of a query
-        use :meth:`pymongo.cursor.Cursor.distinct`.
+        To get the distinct values for a key in the result set of a
+        query use :meth:`~pymongo.cursor.Cursor.distinct`.
 
         :Parameters:
           - `key`: name of key for which we want to get the distinct values
@@ -795,7 +748,7 @@ class Collection(object):
                        ("map", map), ("reduce", reduce)])
         command.update(**kwargs)
 
-        response = self.__database._command(command)
+        response = self.__database.command(command)
         if full_response:
             return response
         return self.__database[response["result"]]
