@@ -14,7 +14,6 @@
 
 """Cursor class to iterate over Mongo query results."""
 
-import types
 import struct
 import warnings
 
@@ -22,7 +21,7 @@ import helpers
 import message
 from son import SON
 from code import Code
-from errors import InvalidOperation, OperationFailure, AutoReconnect
+from errors import InvalidOperation, AutoReconnect
 
 _QUERY_OPTIONS = {
     "tailable_cursor": 2,
@@ -38,7 +37,7 @@ class Cursor(object):
 
     def __init__(self, collection, spec, fields, skip, limit, slave_okay,
                  timeout, tailable, snapshot=False,
-                 _sock=None, _must_use_master=False):
+                 _sock=None, _must_use_master=False, _is_command=False):
         """Create a new cursor.
 
         Should not be called directly by application developers.
@@ -57,6 +56,7 @@ class Cursor(object):
         self.__hint = None
         self.__socket = _sock
         self.__must_use_master = _must_use_master
+        self.__is_command = _is_command
 
         self.__data = []
         self.__id = None
@@ -125,6 +125,8 @@ class Cursor(object):
     def __query_spec(self):
         """Get the spec to use for a query.
         """
+        if self.__is_command:
+            return self.__spec
         spec = SON({"query": self.__spec})
         if self.__ordering:
             spec["orderby"] = self.__ordering
@@ -164,7 +166,7 @@ class Cursor(object):
         :Parameters:
           - `limit`: the number of results to return
         """
-        if not isinstance(limit, types.IntType):
+        if not isinstance(limit, int):
             raise TypeError("limit must be an int")
         self.__check_okay_to_chain()
 
@@ -181,7 +183,7 @@ class Cursor(object):
         :Parameters:
           - `skip`: the number of results to skip
         """
-        if not isinstance(skip, (types.IntType, types.LongType)):
+        if not isinstance(skip, (int, long)):
             raise TypeError("skip must be an int")
         self.__check_okay_to_chain()
 
@@ -218,7 +220,7 @@ class Cursor(object):
           - `index`: An integer or slice index to be applied to this cursor
         """
         self.__check_okay_to_chain()
-        if isinstance(index, types.SliceType):
+        if isinstance(index, slice):
             if index.step is not None:
                 raise IndexError("Cursor instances do not support slice steps")
 
@@ -239,7 +241,7 @@ class Cursor(object):
             self.__limit = limit
             return self
 
-        if isinstance(index, (types.IntType, types.LongType)):
+        if isinstance(index, (int, long)):
             if index < 0:
                 raise IndexError("Cursor instances do not support negative indices")
             clone = self.clone()
@@ -304,18 +306,17 @@ class Cursor(object):
             if self.__skip:
                 command["skip"] = self.__skip
 
-        response = self.__collection.database._command(command,
-                                                       ["ns missing"])
+        response = self.__collection.database.command(command, ["ns missing"])
         if response.get("errmsg", "") == "ns missing":
             return 0
         return int(response["n"])
 
     def distinct(self, key):
-        """Get a list of distinct values for `key` among all documents in the
-        result set of this query.
+        """Get a list of distinct values for `key` among all documents
+        in the result set of this query.
 
         Raises :class:`TypeError` if `key` is not an instance of
-        ``(str, unicode)``.
+        :class:`basestring`.
 
         :Parameters:
           - `key`: name of key for which we want to get the distinct values
@@ -326,15 +327,15 @@ class Cursor(object):
 
         .. versionadded:: 1.2
         """
-        if not isinstance(key, types.StringTypes):
-            raise TypeError("key must be an instance of (str, unicode)")
+        if not isinstance(key, basestring):
+            raise TypeError("key must be an instance of basestring")
 
         command = SON([("distinct", self.__collection.name), ("key", key)])
 
         if self.__spec:
             command["query"] = self.__spec
 
-        return self.__collection.database._command(command)["values"]
+        return self.__collection.database.command(command)["values"]
 
     def explain(self):
         """Returns an explain plan record for this cursor.
@@ -373,7 +374,7 @@ class Cursor(object):
             self.__hint = None
             return self
 
-        if not isinstance(index, (types.ListType)):
+        if not isinstance(index, (list)):
             raise TypeError("hint takes a list specifying an index")
         self.__hint = helpers._index_document(index)
         return self
@@ -381,15 +382,19 @@ class Cursor(object):
     def where(self, code):
         """Adds a $where clause to this query.
 
-        The `code` argument must be an instance of (str, unicode, Code)
-        containing a JavaScript expression. This expression will be evaluated
-        for each object scanned. Only those objects for which the expression
-        evaluates to *true* will be returned as results. The keyword *this*
-        refers to the object currently being scanned.
+        The `code` argument must be an instance of :class:`basestring`
+        or :class:`~pymongo.code.Code` containing a JavaScript
+        expression. This expression will be evaluated for each
+        document scanned. Only those documents for which the
+        expression evaluates to *true* will be returned as
+        results. The keyword *this* refers to the object currently
+        being scanned.
 
-        Raises TypeError if `code` is not an instance of (str, unicode). Raises
-        InvalidOperation if this cursor has already been used. Only the last
-        where clause applied to a cursor has any effect.
+        Raises :class:`TypeError` if `code` is not an instance of
+        :class:`basestring`. Raises
+        :class:`~pymongo.errors.InvalidOperation` if this
+        :class:`Cursor` has already been used. Only the last call to
+        :meth:`where` applied to a :class:`Cursor` has any effect.
 
         :Parameters:
           - `code`: JavaScript expression to use as a filter
@@ -413,7 +418,7 @@ class Cursor(object):
         response = db.connection._send_message_with_response(message,
                                                              **kwargs)
 
-        if isinstance(response, types.TupleType):
+        if isinstance(response, tuple):
             (connection_id, response) = response
         else:
             connection_id = None
