@@ -14,7 +14,7 @@
 
 """Database level operations."""
 
-import types
+import warnings
 try:
     import hashlib
     _md5func = hashlib.md5
@@ -38,15 +38,18 @@ class Database(object):
     def __init__(self, connection, name):
         """Get a database by connection and name.
 
-        Raises TypeError if name is not an instance of (str, unicode). Raises
-        InvalidName if name is not a valid database name.
+        Raises :class:`TypeError` if `name` is not an instance of
+        :class:`basestring`. Raises
+        :class:`~pymongo.errors.InvalidName` if `name` is not a valid
+        database name.
 
         :Parameters:
-          - `connection`: a connection to Mongo
+          - `connection`: a :class:`~pymongo.connection.Connection`
+            instance
           - `name`: database name
         """
-        if not isinstance(name, types.StringTypes):
-            raise TypeError("name must be an instance of (str, unicode)")
+        if not isinstance(name, basestring):
+            raise TypeError("name must be an instance of basestring")
 
         self.__check_name(name)
 
@@ -194,10 +197,27 @@ class Database(object):
         return son
 
     def _command(self, command, allowable_errors=[], check=True, sock=None):
-        """Issue a DB command.
+        warnings.warn("The '_command' method is deprecated. "
+                      "Please use 'command' instead.", DeprecationWarning)
+        return self.command(command, check, allowable_errors, sock)
+
+    def command(self, command, check=True, allowable_errors=[], _sock=None):
+        """Issue a MongoDB command.
+
+        Send a command to the database and return the response.
+
+        :Parameters:
+          - `command`: document representing the command to be issued
+          - `check` (optional): check the response for errors, raising
+            :class:`~pymongo.errors.OperationFailure` if there are any
+          - `allowable_errors`: if `check` is ``True``, error messages in this
+            list will be ignored by error-checking
+
+        .. versionadded:: 1.4
         """
-        result = self["$cmd"].find_one(command, _sock=sock,
-                                       _must_use_master=True)
+        result = self["$cmd"].find_one(command, _sock=_sock,
+                                       _must_use_master=True,
+                                       _is_command=True)
 
         if check and result["ok"] != 1:
             if result["errmsg"] in allowable_errors:
@@ -227,7 +247,7 @@ class Database(object):
         if isinstance(name, Collection):
             name = name.name
 
-        if not isinstance(name, types.StringTypes):
+        if not isinstance(name, basestring):
             raise TypeError("name_or_collection must be an instance of "
                             "(Collection, str, unicode)")
 
@@ -236,7 +256,7 @@ class Database(object):
         if name not in self.collection_names():
             return
 
-        self._command({"drop": unicode(name)})
+        self.command({"drop": unicode(name)})
 
     def validate_collection(self, name_or_collection):
         """Validate a collection.
@@ -248,11 +268,11 @@ class Database(object):
         if isinstance(name, Collection):
             name = name.name
 
-        if not isinstance(name, types.StringTypes):
+        if not isinstance(name, basestring):
             raise TypeError("name_or_collection must be an instance of "
                             "(Collection, str, unicode)")
 
-        result = self._command({"validate": unicode(name)})
+        result = self.command({"validate": unicode(name)})
 
         info = result["result"]
         if info.find("exception") != -1 or info.find("corrupt") != -1:
@@ -265,7 +285,7 @@ class Database(object):
         Returns one of (:data:`~pymongo.OFF`,
         :data:`~pymongo.SLOW_ONLY`, :data:`~pymongo.ALL`).
         """
-        result = self._command({"profile": -1})
+        result = self.command({"profile": -1})
 
         assert result["was"] >= 0 and result["was"] <= 2
         return result["was"]
@@ -280,10 +300,10 @@ class Database(object):
         :Parameters:
           - `level`: the profiling level to use
         """
-        if not isinstance(level, types.IntType) or level < 0 or level > 2:
+        if not isinstance(level, int) or level < 0 or level > 2:
             raise ValueError("level must be one of (OFF, SLOW_ONLY, ALL)")
 
-        self._command({"profile": level})
+        self.command({"profile": level})
 
     def profiling_info(self):
         """Returns a list containing current profiling information.
@@ -296,7 +316,7 @@ class Database(object):
         Return None if the last operation was error-free. Otherwise return the
         error that occurred.
         """
-        error = self._command({"getlasterror": 1})
+        error = self.command({"getlasterror": 1})
         if error.get("err", 0) is None:
             return None
         if error["err"] == "not master":
@@ -308,7 +328,7 @@ class Database(object):
 
         Returns a SON object with status information.
         """
-        return self._command({"getlasterror": 1})
+        return self.command({"getlasterror": 1})
 
     def previous_error(self):
         """Get the most recent error to have occurred on this database.
@@ -317,7 +337,7 @@ class Database(object):
         `Database.reset_error_history`. Returns None if no such errors have
         occurred.
         """
-        error = self._command({"getpreverror": 1})
+        error = self.command({"getpreverror": 1})
         if error.get("err", 0) is None:
             return None
         return error
@@ -328,7 +348,7 @@ class Database(object):
         Calls to `Database.previous_error` will only return errors that have
         occurred since the most recent call to this method.
         """
-        self._command({"reseterror": 1})
+        self.command({"reseterror": 1})
 
     def __iter__(self):
         return self
@@ -339,47 +359,103 @@ class Database(object):
     def _password_digest(self, username, password):
         """Get a password digest to use for authentication.
         """
-        if not isinstance(password, types.StringTypes):
-            raise TypeError("password must be an instance of (str, unicode)")
-        if not isinstance(username, types.StringTypes):
-            raise TypeError("username must be an instance of (str, unicode)")
+        if not isinstance(password, basestring):
+            raise TypeError("password must be an instance of basestring")
+        if not isinstance(username, basestring):
+            raise TypeError("username must be an instance of basestring")
 
         md5hash = _md5func()
-        md5hash.update(username + ":mongo:" + password)
+        md5hash.update(username.encode('utf-8') + ":mongo:" + password.encode('utf-8'))
         return unicode(md5hash.hexdigest())
 
+    def add_user(self, name, password):
+        """Create user `name` with password `password`.
+
+        Add a new user with permissions for this :class:`Database`.
+
+        .. note:: Will change the password if user `name` already exists.
+
+        :Parameters:
+          - `name`: the name of the user to create
+          - `password`: the password of the user to create
+
+        .. versionadded:: 1.4
+        """
+        self.system.users.update({"user": name},
+                                 {"user": name,
+                                  "pwd": self._password_digest(name, password)},
+                                 upsert=True, safe=True)
+
+    def remove_user(self, name):
+        """Remove user `name` from this :class:`Database`.
+
+        User `name` will no longer have permissions to access this
+        :class:`Database`.
+
+        :Paramaters:
+          - `name`: the name of the user to remove
+
+        .. versionadded:: 1.4
+        """
+        self.system.users.remove({"user": name}, safe=True)
+
     def authenticate(self, name, password):
         """Authenticate to use this database.
 
-        Once authenticated, the user has full read and write access to this
-        database. Raises TypeError if either name or password is not an
-        instance of (str, unicode). Authentication lasts for the life of the
-        database connection, or until `Database.logout` is called.
+        Once authenticated, the user has full read and write access to
+        this database. Raises :class:`TypeError` if either `name` or
+        `password` is not an instance of ``(str,
+        unicode)``. Authentication lasts for the life of the database
+        connection, or until :meth:`logout` is called.
 
-        The "admin" database is special. Authenticating on "admin" gives access
-        to *all* databases. Effectively, "admin" access means root access to
-        the database.
+        The "admin" database is special. Authenticating on "admin"
+        gives access to *all* databases. Effectively, "admin" access
+        means root access to the database.
+
+        .. note:: Currently, authentication is per
+           :class:`~socket.socket`. This means that there are a couple
+           of situations in which re-authentication is necessary:
+
+           - On failover (when an
+             :class:`~pymongo.errors.AutoReconnect` exception is
+             raised).
+
+           - After a call to
+             :meth:`~pymongo.connection.Connection.disconnect` or
+             :meth:`~pymongo.connection.Connection.end_request`.
+
+           - When sharing a :class:`~pymongo.connection.Connection`
+             between multiple threads, each thread will need to
+             authenticate separately.
+
+        .. warning:: Currently, calls to
+           :meth:`~pymongo.connection.Connection.end_request` will
+           lead to unpredictable behavior in combination with
+           auth. The :class:`~socket.socket` owned by the calling
+           thread will be returned to the pool, so whichever thread
+           uses that :class:`~socket.socket` next will have whatever
+           permissions were granted to the calling thread.
 
         :Parameters:
           - `name`: the name of the user to authenticate
           - `password`: the password of the user to authenticate
         """
-        if not isinstance(name, types.StringTypes):
-            raise TypeError("name must be an instance of (str, unicode)")
-        if not isinstance(password, types.StringTypes):
-            raise TypeError("password must be an instance of (str, unicode)")
+        if not isinstance(name, basestring):
+            raise TypeError("name must be an instance of basestring")
+        if not isinstance(password, basestring):
+            raise TypeError("password must be an instance of basestring")
 
-        result = self._command({"getnonce": 1})
+        result = self.command({"getnonce": 1})
         nonce = result["nonce"]
         digest = self._password_digest(name, password)
         md5hash = _md5func()
         md5hash.update("%s%s%s" % (nonce, unicode(name), digest))
         key = unicode(md5hash.hexdigest())
         try:
-            result = self._command(SON([("authenticate", 1),
-                                        ("user", unicode(name)),
-                                        ("nonce", nonce),
-                                        ("key", key)]))
+            result = self.command(SON([("authenticate", 1),
+                                       ("user", unicode(name)),
+                                       ("nonce", nonce),
+                                       ("key", key)]))
             return True
         except OperationFailure:
             return False
@@ -389,7 +465,7 @@ class Database(object):
 
         Note that other databases may still be authorized.
         """
-        self._command({"logout": 1})
+        self.command({"logout": 1})
 
     def dereference(self, dbref):
         """Dereference a DBRef, getting the SON object it points to.
@@ -432,7 +508,7 @@ class Database(object):
             code = Code(code)
 
         command = SON([("$eval", code), ("args", list(args))])
-        result = self._command(command)
+        result = self.command(command)
         return result.get("retval", None)
 
     def __call__(self, *args, **kwargs):
