@@ -12,10 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Exceptions raised by the Mongo driver."""
+"""Exceptions raised by PyMongo."""
+
+class PyMongoError(Exception):
+    """Base class for all PyMongo exceptions.
+
+    .. versionadded:: 1.4
+    """
 
 
-class ConnectionFailure(IOError):
+class ConnectionFailure(PyMongoError):
     """Raised when a connection to the database cannot be made or is lost.
     """
 
@@ -32,50 +38,53 @@ class AutoReconnect(ConnectionFailure):
     """
 
 
-class ConfigurationError(Exception):
+class ConfigurationError(PyMongoError):
     """Raised when something is incorrectly configured.
     """
 
 
-class OperationFailure(Exception):
+class OperationFailure(PyMongoError):
     """Raised when a database operation fails.
     """
 
 
-class InvalidOperation(Exception):
+class DuplicateKeyError(OperationFailure):
+    """Raised when a safe insert or update fails due to a duplicate key error.
+
+    .. versionadded:: 1.4
+    """
+
+
+class InvalidOperation(PyMongoError):
     """Raised when a client attempts to perform an invalid operation.
     """
 
 
-class CollectionInvalid(Exception):
+class CollectionInvalid(PyMongoError):
     """Raised when collection validation fails.
     """
 
 
-class InvalidName(ValueError):
+class InvalidName(PyMongoError):
     """Raised when an invalid name is used.
     """
 
 
-class InvalidBSON(ValueError):
+class InvalidBSON(PyMongoError):
     """Raised when trying to create a BSON object from invalid data.
     """
 
-class InvalidStringData(ValueError):
+
+class InvalidStringData(PyMongoError):
     """Raised when trying to encode a string containing non-UTF8 data.
     """
 
 
-class InvalidDocument(ValueError):
+class InvalidDocument(PyMongoError):
     """Raised when trying to create a BSON object from an invalid document.
     """
 
 
-class UnsupportedTag(ValueError):
-    """Raised when trying to parse an unsupported tag in an XML document.
-    """
-
-
-class InvalidId(ValueError):
+class InvalidId(PyMongoError):
     """Raised when trying to create an ObjectId from invalid data.
     """
