@@ -17,7 +17,6 @@
 Performs all writes to Master instance and distributes reads among all
 instances."""
 
-import types
 import random
 
 from database import Database
@@ -37,11 +36,6 @@ class MasterSlaveConnection(object):
         connection pooling, etc. 'Connection' instances used as slaves should
         be created with the slave_okay option set to True.
 
-        If connection pooling is being used the connections should be created
-        with "auto_start_request" mode set to False. All request functionality
-        that is needed should be initiated by calling `start_request` on the
-        `MasterSlaveConnection` instance.
-
         Raises TypeError if `master` is not an instance of `Connection` or
         slaves is not a list of at least one `Connection` instances.
 
@@ -52,7 +46,7 @@ class MasterSlaveConnection(object):
         """
         if not isinstance(master, Connection):
             raise TypeError("master must be a Connection instance")
-        if not isinstance(slaves, types.ListType) or len(slaves) == 0:
+        if not isinstance(slaves, list) or len(slaves) == 0:
             raise TypeError("slaves must be a list of length >= 1")
 
         for slave in slaves:
@@ -93,7 +87,7 @@ class MasterSlaveConnection(object):
     # _connection_to_use is a hack that we need to include to make sure
     # that killcursor operations can be sent to the same instance on which
     # the cursor actually resides...
-    def _send_message(self, operation, data, safe=False, _connection_to_use=None):
+    def _send_message(self, message, safe=False, _connection_to_use=None):
         """Say something to Mongo.
 
         Sends a message on the Master connection. This is used for inserts,
@@ -108,13 +102,13 @@ class MasterSlaveConnection(object):
           - `safe`: perform a getLastError after sending the message
         """
         if _connection_to_use is None or _connection_to_use == -1:
-            return self.__master._send_message(operation, data, safe)
-        return self.__slaves[_connection_to_use]._send_message(operation, data, safe)
+            return self.__master._send_message(message, safe)
+        return self.__slaves[_connection_to_use]._send_message(message, safe)
 
     # _connection_to_use is a hack that we need to include to make sure
     # that getmore operations can be sent to the same instance on which
     # the cursor actually resides...
-    def _send_message_with_response(self, operation, data,
+    def _send_message_with_response(self, message,
                                     _sock=None, _connection_to_use=None,
                                     _must_use_master=False):
         """Receive a message from Mongo.
@@ -127,13 +121,12 @@ class MasterSlaveConnection(object):
         """
         if _connection_to_use is not None:
             if _connection_to_use == -1:
-                return (-1, self.__master._send_message_with_response(operation,
-                                                                      data,
+                return (-1, self.__master._send_message_with_response(message,
                                                                       _sock))
             else:
                 return (_connection_to_use,
                         self.__slaves[_connection_to_use]
-                        ._send_message_with_response(operation, data, _sock))
+                        ._send_message_with_response(message, _sock))
 
         # for now just load-balance randomly among slaves only...
         connection_id = random.randrange(0, len(self.__slaves))
@@ -142,23 +135,21 @@ class MasterSlaveConnection(object):
         # master instance. any queries in a request must be sent to the
         # master since that is where writes go.
         if _must_use_master or self.__in_request or connection_id == -1:
-            return (-1, self.__master._send_message_with_response(operation,
-                                                                  data, _sock))
+            return (-1, self.__master._send_message_with_response(message,
+                                                                  _sock))
 
         return (connection_id,
-                self.__slaves[connection_id]._send_message_with_response(operation,
-                                                                         data,
+                self.__slaves[connection_id]._send_message_with_response(message,
                                                                          _sock))
 
     def start_request(self):
         """Start a "request".
 
-        See documentation for `Connection.start_request`. Note that all
-        operations performed within a request will be sent using the Master
-        connection.
+        Start a sequence of operations in which order matters. Note
+        that all operations performed within a request will be sent
+        using the Master connection.
         """
         self.__in_request = True
-        self.__master.start_request()
 
     def end_request(self):
         """End the current "request".
