@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Copyright 2009 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +15,7 @@
 # limitations under the License.
 
 """Test the collection module."""
+
 import warnings
 import unittest
 import re
@@ -31,6 +34,7 @@ from pymongo.code import Code
 from pymongo.binary import Binary
 from pymongo.collection import Collection
 from pymongo.errors import InvalidName, OperationFailure, InvalidDocument
+from pymongo.errors import DuplicateKeyError
 from pymongo import ASCENDING, DESCENDING
 from pymongo.son import SON
 
@@ -57,6 +61,7 @@ class TestCollection(unittest.TestCase):
         self.assertRaises(InvalidName, make_col, self.db.test, ".test")
         self.assertRaises(InvalidName, make_col, self.db.test, "test.")
         self.assertRaises(InvalidName, make_col, self.db.test, "tes..t")
+        self.assertRaises(InvalidName, make_col, self.db.test, "tes\x00t")
 
         self.assert_(isinstance(self.db.test, Collection))
         self.assertEqual(self.db.test, self.db["test"])
@@ -462,6 +467,26 @@ class TestCollection(unittest.TestCase):
         db.test.save({"hello": "world"})
         self.assert_(db.error())
 
+    def test_duplicate_key_error(self):
+        db = self.db
+        db.drop_collection("test")
+
+        db.test.create_index("x", unique=True)
+
+        db.test.insert({"_id": 1, "x": 1}, safe=True)
+        db.test.insert({"_id": 2, "x": 2}, safe=True)
+
+        self.assertRaises(DuplicateKeyError,
+                          db.test.insert, {"_id": 1}, safe=True)
+        self.assertRaises(DuplicateKeyError,
+                          db.test.insert, {"x": 1}, safe=True)
+
+        self.assertRaises(DuplicateKeyError,
+                          db.test.save, {"x": 2}, safe=True)
+        self.assertRaises(DuplicateKeyError,
+                          db.test.update, {"x": 1}, {"$inc": {"x": 1}}, safe=True)
+
+
     def test_index_on_subfield(self):
         db = self.db
         db.drop_collection("test")
@@ -520,6 +545,9 @@ class TestCollection(unittest.TestCase):
         for doc in db.test.find():
             self.assertEqual(5, doc["y"])
 
+        self.assertEqual(2, db.test.update({"x": 4}, {"$set": {"y": 6}},
+                                           multi=True, safe=True)["n"])
+
     def test_upsert(self):
         db = self.db
         db.drop_collection("test")
@@ -540,7 +568,7 @@ class TestCollection(unittest.TestCase):
         db.test.insert({"x": 5})
         id = db.test.insert({"x": 4})
 
-        db.test.update({"_id": id}, {"$inc": {"x": 1}})
+        self.assertEqual(None, db.test.update({"_id": id}, {"$inc": {"x": 1}}))
         if v113minus:
             self.assert_(db.error()["err"].startswith("E11001"))
         else:
@@ -549,6 +577,10 @@ class TestCollection(unittest.TestCase):
         self.assertRaises(OperationFailure, db.test.update,
                           {"_id": id}, {"$inc": {"x": 1}}, safe=True)
 
+        self.assertEqual(1, db.test.update({"_id": id}, {"$inc": {"x": 2}}, safe=True)["n"])
+
+        self.assertEqual(0, db.test.update({"_id": "foo"}, {"$inc": {"x": 2}}, safe=True)["n"])
+
     def test_safe_save(self):
         db = self.db
         db.drop_collection("test")
@@ -568,7 +600,7 @@ class TestCollection(unittest.TestCase):
         db.test.insert({"x": 1})
         self.assertEqual(1, db.test.count())
 
-        db.test.remove({"x": 1})
+        self.assertEqual(None, db.test.remove({"x": 1}))
         self.assertEqual(1, db.test.count())
 
         if version.at_least(db.connection, (1, 1, 3, -1)):
@@ -576,6 +608,12 @@ class TestCollection(unittest.TestCase):
         else: # Just test that it doesn't blow up
             db.test.remove({"x": 1}, safe=True)
 
+        db.drop_collection("test")
+        db.test.insert({"x": 1})
+        db.test.insert({"x": 1})
+        self.assertEqual(2, db.test.remove({}, safe=True)["n"])
+        self.assertEqual(0, db.test.remove({}, safe=True)["n"])
+
     def test_count(self):
         db = self.db
         db.drop_collection("test")
@@ -594,59 +632,54 @@ class TestCollection(unittest.TestCase):
             self.assertEqual(eval, expected)
 
 
-        args = [[], {},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }"]
-        expected = []
-        group_checker(args, expected)
+        self.assertEqual([], db.test.group([], {}, {"count": 0},
+                                           "function (obj, prev) { prev.count++; }"))
 
         db.test.save({"a": 2})
         db.test.save({"b": 5})
         db.test.save({"a": 1})
 
-        args = [[], {},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }"]
-        expected = [{'count': 3}]
-        group_checker(args, expected)
+        self.assertEqual([{"count": 3}],
+                         db.test.group([], {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }"))
 
-        args = [[],
-                {"a": {"$gt": 1}},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }"]
-        expected = [{'count': 1}]
-        group_checker(args, expected)
+        self.assertEqual([{"count": 1}],
+                         db.test.group([], {"a": {"$gt": 1}}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }"))
 
         db.test.save({"a": 2, "b": 3})
 
-        args = [["a"], {},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }"]
-        # NOTE maybe we can't count on this ordering being right
-        expected = [{"a": 2, "count": 2},
-                    {"a": None, "count": 1},
-                    {"a": 1, "count": 1}]
-        group_checker(args, expected)
+        self.assertEqual([{"a": 2, "count": 2},
+                          {"a": None, "count": 1},
+                          {"a": 1, "count": 1}],
+                         db.test.group(["a"], {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }"))
 
         # modifying finalize
-        args = [["a"], {},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }",
-                "function(obj){obj.count++;}"]
-        expected = [{"a": 2, "count": 3},
-                    {"a": None, "count": 2},
-                    {"a": 1, "count": 2}]
-        group_checker(args, expected)
+        self.assertEqual([{"a": 2, "count": 3},
+                          {"a": None, "count": 2},
+                          {"a": 1, "count": 2}],
+                         db.test.group(["a"], {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }",
+                                       "function (obj) { obj.count++; }"))
 
         # returning finalize
-        args = [["a"], {},
-                {"count": 0},
-                "function (obj, prev) { prev.count++; }",
-                "function(obj){ return obj.count;}"]
-        expected = [2, # a:2
-                    1, # a:None
-                    1] # a:1
-        group_checker(args, expected)
+        self.assertEqual([2, 1, 1],
+                         db.test.group(["a"], {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }",
+                                       "function (obj) { return obj.count; }"))
+
+        # keyf
+        self.assertEqual([2, 2],
+                         db.test.group("function (obj) { if (obj.a == 2) { return {a: true} }; "
+                                       "return {b: true}; }", {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }",
+                                       "function (obj) { return obj.count; }"))
+
+        # no key
+        self.assertEqual([{"count": 4}],
+                         db.test.group(None, {}, {"count": 0},
+                                       "function (obj, prev) { prev.count++; }"))
 
         warnings.simplefilter("error")
         self.assertRaises(DeprecationWarning,
@@ -887,6 +920,14 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(1, result.find_one({"_id": "dog"})["value"])
         self.assertEqual(None, result.find_one({"_id": "mouse"}))
 
+    def test_messages_with_unicode_collection_names(self):
+        db = self.db
+
+        db[u"Employés"].insert({"x": 1})
+        db[u"Employés"].update({"x": 1}, {"x": 2})
+        db[u"Employés"].remove({})
+        db[u"Employés"].find_one()
+        list(db[u"Employés"].find())
 
 if __name__ == "__main__":
     unittest.main()
