@@ -14,7 +14,6 @@
 
 """Test the cursor module."""
 import unittest
-import types
 import random
 import warnings
 import sys
@@ -88,20 +87,6 @@ class TestCursor(unittest.TestCase):
         self.assertRaises(TypeError, db.test.find().hint, index)
 
 
-    # This is deprecated - test that a warning is actually raised
-    def test_slave_okay(self):
-        db = self.db
-        db.drop_collection("test")
-        db.test.save({"x": 1})
-
-        warnings.simplefilter("error")
-
-        self.assertEqual(1, db.test.find().next()["x"])
-        self.assertRaises(DeprecationWarning, db.test.find, slave_okay=True)
-        self.assertRaises(DeprecationWarning, db.test.find, slave_okay=False)
-
-        warnings.simplefilter("default")
-
     def test_limit(self):
         db = self.db
 
@@ -258,7 +243,7 @@ class TestCursor(unittest.TestCase):
             db.test.save({"x": i})
 
         self.assertEqual(10, db.test.find().count())
-        self.assert_(isinstance(db.test.find().count(), types.IntType))
+        self.assert_(isinstance(db.test.find().count(), int))
         self.assertEqual(10, db.test.find().limit(5).count())
         self.assertEqual(10, db.test.find().skip(5).count())
 
@@ -318,34 +303,34 @@ class TestCursor(unittest.TestCase):
         db = self.db
         db.drop_collection("test")
 
-        client_cursors = db._command({"cursorInfo": 1})["clientCursors_size"]
-        by_location = db._command({"cursorInfo": 1})["byLocation_size"]
+        client_cursors = db.command({"cursorInfo": 1})["clientCursors_size"]
+        by_location = db.command({"cursorInfo": 1})["byLocation_size"]
 
         test = db.test
         for i in range(10000):
             test.insert({"i": i})
 
         self.assertEqual(client_cursors,
-                         db._command({"cursorInfo": 1})["clientCursors_size"])
+                         db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertEqual(by_location,
-                         db._command({"cursorInfo": 1})["byLocation_size"])
+                         db.command({"cursorInfo": 1})["byLocation_size"])
 
         for _ in range(10):
             db.test.find_one()
 
         self.assertEqual(client_cursors,
-                         db._command({"cursorInfo": 1})["clientCursors_size"])
+                         db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertEqual(by_location,
-                         db._command({"cursorInfo": 1})["byLocation_size"])
+                         db.command({"cursorInfo": 1})["byLocation_size"])
 
         for _ in range(10):
             for x in db.test.find():
                 break
 
         self.assertEqual(client_cursors,
-                         db._command({"cursorInfo": 1})["clientCursors_size"])
+                         db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertEqual(by_location,
-                         db._command({"cursorInfo": 1})["byLocation_size"])
+                         db.command({"cursorInfo": 1})["byLocation_size"])
 
         a = db.test.find()
         for x in a:
@@ -353,25 +338,25 @@ class TestCursor(unittest.TestCase):
 
         self.assertNotEqual(
             client_cursors,
-            db._command({"cursorInfo": 1})["clientCursors_size"])
+            db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertNotEqual(by_location,
-                            db._command({"cursorInfo": 1})["byLocation_size"])
+                            db.command({"cursorInfo": 1})["byLocation_size"])
 
         del a
 
         self.assertEqual(client_cursors,
-                         db._command({"cursorInfo": 1})["clientCursors_size"])
+                         db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertEqual(by_location,
-                         db._command({"cursorInfo": 1})["byLocation_size"])
+                         db.command({"cursorInfo": 1})["byLocation_size"])
 
         a = db.test.find().limit(10)
         for x in a:
             break
 
         self.assertEqual(client_cursors,
-                         db._command({"cursorInfo": 1})["clientCursors_size"])
+                         db.command({"cursorInfo": 1})["clientCursors_size"])
         self.assertEqual(by_location,
-                         db._command({"cursorInfo": 1})["byLocation_size"])
+                         db.command({"cursorInfo": 1})["byLocation_size"])
 
     def test_rewind(self):
         self.db.test.save({"x": 1})
