@@ -20,7 +20,7 @@ import time
 from pymongo.errors import ConnectionFailure
 from pymongo.connection import Connection
 
-db = Connection.paired(("localhost", 27018), pool_size=10).test
+db = Connection.paired(("localhost", 27018)).test
 db.test.remove({})
 
 class Something(threading.Thread):
@@ -31,7 +31,7 @@ class Something(threading.Thread):
                 id = db.test.save({"x": 1})
                 assert db.test.find_one(id)["x"] == 1
                 db.test.remove(id)
-                db.connection().end_request()
+                db.connection.end_request()
                 print "Y"
             except ConnectionFailure, e:
                 print e
